/*
** Copyright (C) 2003-2006 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
**
** Use of the SILK system and related source code is subject to the terms
** of the following licenses:
**
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
**
** NO WARRANTY
**
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE,
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT,
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES.
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE
** DELIVERABLES UNDER THIS LICENSE.
**
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie
** Mellon University, its trustees, officers, employees, and agents from
** all claims or demands made against them (and any related losses,
** expenses, or attorney's fees) arising out of, or relating to Licensee's
** and/or its sub licensees' negligent use or willful misuse of or
** negligent conduct or willful misconduct regarding the Software,
** facilities, or other rights or assistance granted by Carnegie Mellon
** University under this License, including, but not limited to, any
** claims of product liability, personal injury, death, damage to
** property, or violation of any laws or regulations.
**
** Carnegie Mellon University Software Engineering Institute authored
** documents are sponsored by the U.S. Department of Defense under
** Contract F19628-00-C-0003. Carnegie Mellon University retains
** copyrights in all material produced under this contract. The U.S.
** Government retains a non-exclusive, royalty-free license to publish or
** reproduce these documents, or allow others to do so, for U.S.
** Government purposes only pursuant to the copyright license under the
** contract clause at 252.227.7013.
**
** @OPENSOURCE_HEADER_END@
*/

#if !defined(_SILK_SITE_H) || defined(_SILK_SITE_TWOWAY_H)
#error "Never include silk_site_twoway.h directly, include silk_site.h instead"
#endif
#define _SILK_SITE_TWOWAY_H

RCSIDENTVAR(rcsID_SILK_SITE_TWOWAY_H, "$SiLK: silk_site_twoway.h 5842 2006-12-04 17:27:10Z mthomas $");


/*
 *  Whether to split the web data separately from the other data; that
 *  is, whether to use the "inweb" and "outweb" flowtypes.
 *
 *  Web data is any flow seen on ports 80/tcp, 8080/tcp, 443/tcp.
 */
#define SK_ENABLE_WEB_SPLIT   1

/*
 *  Whether to split ICMP data separately from the other data; that
 *  is, whether to use the "inicmp" and "outicmp" flowtypes.
 *
 *  ICMP data is any flow where proto == 1.
 */
#define SK_ENABLE_ICMP_SPLIT  0


#ifdef DEFINE_SITE_VARIABLES

#ifdef SK_ENABLE_INITIAL_TCPFLAGS
#  define FILE_FORMAT_NONWEB FT_RWAUGMENTED
#  define FILE_FORMAT_WEB    FT_RWAUGWEB
#else
#  define FILE_FORMAT_NONWEB FT_RWSPLIT
#  define FILE_FORMAT_WEB    FT_RWWWW
#endif

/* Define the file formats used to pack each classtype.  If these do not
 * line up with the type IDs defined in the config file, there will
 * be problems.
 */

uint32_t sksite_filetypeFormats[] = {
    /* in      */  FILE_FORMAT_NONWEB,
    /* out     */  FILE_FORMAT_NONWEB,
    /* inweb   */  FILE_FORMAT_WEB,
    /* outweb  */  FILE_FORMAT_WEB,
    /* innull  */  FILE_FORMAT_NONWEB,
    /* outnull */  FILE_FORMAT_NONWEB,
    /* int2int */  FILE_FORMAT_NONWEB,
    /* ext2ext */  FILE_FORMAT_NONWEB,
    /* inicmp  */  FILE_FORMAT_NONWEB,
    /* outicmp */  FILE_FORMAT_NONWEB,
    /* other   */  FILE_FORMAT_NONWEB
};

#endif /* DEFINE_SITE_VARIABLES */

/*
** Local variables:
** mode:c
** indent-tabs-mode:nil
** c-basic-offset:4
** End:
*/
