/*
** Copyright (C) 2004-2006 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
**
** Use of the SILK system and related source code is subject to the terms
** of the following licenses:
**
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
**
** NO WARRANTY
**
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE,
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT,
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES.
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE
** DELIVERABLES UNDER THIS LICENSE.
**
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie
** Mellon University, its trustees, officers, employees, and agents from
** all claims or demands made against them (and any related losses,
** expenses, or attorney's fees) arising out of, or relating to Licensee's
** and/or its sub licensees' negligent use or willful misuse of or
** negligent conduct or willful misconduct regarding the Software,
** facilities, or other rights or assistance granted by Carnegie Mellon
** University under this License, including, but not limited to, any
** claims of product liability, personal injury, death, damage to
** property, or violation of any laws or regulations.
**
** Carnegie Mellon University Software Engineering Institute authored
** documents are sponsored by the U.S. Department of Defense under
** Contract F19628-00-C-0003. Carnegie Mellon University retains
** copyrights in all material produced under this contract. The U.S.
** Government retains a non-exclusive, royalty-free license to publish or
** reproduce these documents, or allow others to do so, for U.S.
** Government purposes only pursuant to the copyright license under the
** contract clause at 252.227.7013.
**
** @OPENSOURCE_HEADER_END@
*/
#ifndef _CONVERT_CFLOWD_H
#define _CONVERT_CFLOWD_H

#include "silk.h"

RCSIDENTVAR(rcsID_CONVERT_CFLOWD_H, "$SiLK: convert-cflowd.h 4876 2006-09-14 21:18:03Z mthomas $");


/*
**  convert-cflowd.h
**
**  Handles conversions from netflow V5.
**
*/

#include "rwpack.h"
#include "flowcap.h"

typedef struct _cflowdTimeInfo_t {
    uint32_t rtr_unix_secs;
    uint32_t rtr_unix_msecs;
    intmax_t sysUptime;
} cflowdTimeInfo_t;


/* Encapsulates information from a netflowV5 header.  The resulting
   cflowdTimeInfo_t is used to do time calculations when processing
   individual records. */
void cflowdTimeInfoSetup(const v5Header *hdr, cflowdTimeInfo_t *ti);


/* Fills out a flowcap record from a v5pdu record.  The variable 'ti'
 * can be NULL, in which case the start time will not necessarily
 * reflect reality.  Version is the version of flowcapRec used.
 * Returns -1 on error, 0 on success. */
int cflowdToFlowcap(
    const v5Record     *v5RPtr,
    flowcapRec_t       *fc_rec,
    int                 fc_version,
    cflowdTimeInfo_t   *ti);

/* Fills out a rwRec in a specific byte order from a v5pdu record.
 * The variable 'ti' can be NULL, in which case the start time will
 * not necessarily reflect reality. */
int cflowdToGeneric(
    const v5Record     *v5RPtr,
    rwRec              *rwrec,
    cflowdTimeInfo_t   *ti);


/* Number of milliseconds to adjust the SysUptime when determining
   event ordering.  Routers are producing PDUs which are advertising
   times *before* the flow start and end times. */
extern const intmax_t sysUptimeAdjustMsecs;

#endif /* _CONVERT_CFLOWD_H */


/*
** Local variables:
** mode:c
** indent-tabs-mode:nil
** c-basic-offset:4
** End:
*/


