/*
** Copyright (C) 2004-2006 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
** 
** Use of the SILK system and related source code is subject to the terms 
** of the following licenses:
** 
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
** 
** NO WARRANTY
** 
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
** DELIVERABLES UNDER THIS LICENSE.
** 
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
** Mellon University, its trustees, officers, employees, and agents from 
** all claims or demands made against them (and any related losses, 
** expenses, or attorney's fees) arising out of, or relating to Licensee's 
** and/or its sub licensees' negligent use or willful misuse of or 
** negligent conduct or willful misconduct regarding the Software, 
** facilities, or other rights or assistance granted by Carnegie Mellon 
** University under this License, including, but not limited to, any 
** claims of product liability, personal injury, death, damage to 
** property, or violation of any laws or regulations.
** 
** Carnegie Mellon University Software Engineering Institute authored 
** documents are sponsored by the U.S. Department of Defense under 
** Contract F19628-00-C-0003. Carnegie Mellon University retains 
** copyrights in all material produced under this contract. The U.S. 
** Government retains a non-exclusive, royalty-free license to publish or 
** reproduce these documents, or allow others to do so, for U.S. 
** Government purposes only pursuant to the copyright license under the 
** contract clause at 252.227.7013.
** 
** @OPENSOURCE_HEADER_END@
*/
#ifndef _CCFILTER_PRIV_H
#define _CCFILTER_PRIV_H

#include "silk.h"

RCSIDENTVAR(rcsID_PREFIXTREE_PRIV_H, "$SiLK: ccfilter_priv.h 3970 2006-06-13 17:30:31Z mthomas $");

/*
** ccfilter_priv.h
**
** John McClary Prevost
** December 9th, 2004
**
** Private definitions for ccfilter country code filter plugin.
*/

#include "skprefixmap.h"

/* filter option storage */
#define CCFILTER_VECTOR_BITS 65536
#define CCFILTER_VECTOR_GET_BIT(a,i) ((a[i>>5] & (1 << (i & 0x1F))) != 0)
#define CCFILTER_VECTOR_SET_BIT(a,i) (a[i>>5] |= (1 << (i & 0x1F)))

/* convert an int to a CC */
#define CODE_TO_CC(len, out, out_len, code)                             \
    if ((code) == SKPREFIXMAP_NOT_FOUND) {                              \
        (len) = snprintf((out), (out_len), "??");                       \
    } else {                                                            \
        (len) = snprintf((out), (out_len), "%c%c",                      \
                         (char)(((code)>>8)&0xFF),(char)((code)&0xFF)); \
    }


typedef enum ccFilter_err_e {
    CCFILTER_OK = 0,
    CCFILTER_ERR_MEM = 1,
    CCFILTER_ERR_DATA = 2,
    CCFILTER_ERR_REGISTER = 3,
    CCFILTER_ERR_ARGS = 4
} ccFilter_err_t;

/* data file storage */
#define DEFAULT_CCFILTER_FILENAME "country_codes.pmap"

#endif /* _CCFILTER_PRIV_H */
