/*
** Copyright (C) 2005-2006 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
**
** Use of the SILK system and related source code is subject to the terms
** of the following licenses:
**
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
**
** NO WARRANTY
**
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE,
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT,
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES.
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE
** DELIVERABLES UNDER THIS LICENSE.
**
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie
** Mellon University, its trustees, officers, employees, and agents from
** all claims or demands made against them (and any related losses,
** expenses, or attorney's fees) arising out of, or relating to Licensee's
** and/or its sub licensees' negligent use or willful misuse of or
** negligent conduct or willful misconduct regarding the Software,
** facilities, or other rights or assistance granted by Carnegie Mellon
** University under this License, including, but not limited to, any
** claims of product liability, personal injury, death, damage to
** property, or violation of any laws or regulations.
**
** Carnegie Mellon University Software Engineering Institute authored
** documents are sponsored by the U.S. Department of Defense under
** Contract F19628-00-C-0003. Carnegie Mellon University retains
** copyrights in all material produced under this contract. The U.S.
** Government retains a non-exclusive, royalty-free license to publish or
** reproduce these documents, or allow others to do so, for U.S.
** Government purposes only pursuant to the copyright license under the
** contract clause at 252.227.7013.
**
** @OPENSOURCE_HEADER_END@
*/

/*
**  Test functions for skbitmap.c
**
*/


#include "silk.h"

RCSIDENT("$SiLK: skbitmap-test.c 3983 2006-06-14 18:41:39Z mthomas $");

#include "utils.h"


#define TEST(s)    fprintf(stderr, s "...");
#define RESULT(b)                                                       \
    if ((b)) {                                                          \
        fprintf(stderr, "ok\n");                                        \
    } else {                                                            \
        fprintf(stderr, "failed at %s:%d (rv=%d, i=%u, j=%u, sz=%u)\n", \
                __FILE__, __LINE__, rv, i, j, sz);                      \
        exit(EXIT_FAILURE);                                             \
    }

#define CHECK_MAP(bmap, pos)                            \
    for (sz = 0; sz < (BITMAP_SIZE >> 5); ++sz) {       \
        if (sz == (pos >> 5)) {                         \
            if (bmap->map[sz] == 0) {                   \
                RESULT(0);                              \
            }                                           \
        } else if (bmap->map[sz] != 0) {                \
            RESULT(0);                                  \
        }                                               \
    }


int main()
{
#define BITMAP_SIZE   160
    sk_bitmap_t *bmap;
    uint32_t i = 0;
    uint32_t j = 0;
    int rv = 0xFFFF;
    uint32_t sz;

    TEST("skBitmapCreate");
    rv = skBitmapCreate(&bmap, BITMAP_SIZE);
    CHECK_MAP(bmap, (BITMAP_SIZE * 2));
    RESULT(rv == 0 && bmap != NULL);

    TEST("skBitmapGetSize");
    sz = skBitmapGetSize(bmap);
    RESULT(sz == BITMAP_SIZE);

    TEST("skBitmapGetHighCount");
    sz = skBitmapGetHighCount(bmap);
    RESULT(sz == 0);

    i = 96;

    TEST("skBitmapGetBit");
    rv = skBitmapGetBit(bmap, i);
    RESULT(rv == 0);

    TEST("skBitmapSetBit");
    skBitmapSetBit(bmap, i);
    CHECK_MAP(bmap, i);
    RESULT(1);

    TEST("skBitmapGetBit");
    rv = skBitmapGetBit(bmap, i);
    RESULT(rv == 1);

    TEST("skBitmapGetHighCount");
    sz = skBitmapGetHighCount(bmap);
    RESULT(sz == 1);

    j = 97;

    TEST("skBitmapGetBit");
    rv = skBitmapGetBit(bmap, j);
    RESULT(rv == 0);

    TEST("skBitmapClearBit");
    skBitmapClearBit(bmap, j);
    CHECK_MAP(bmap, i);
    RESULT(1);

    TEST("skBitmapGetBit");
    rv = skBitmapGetBit(bmap, j);
    RESULT(rv == 0);

    TEST("skBitmapGetBit");
    rv = skBitmapGetBit(bmap, i);
    RESULT(rv == 1);

    TEST("skBitmapGetHighCount");
    sz = skBitmapGetHighCount(bmap);
    RESULT(sz == 1);

    TEST("skBitmapSetBit");
    skBitmapSetBit(bmap, j);
    CHECK_MAP(bmap, i);
    RESULT(1);

    TEST("skBitmapGetHighCount");
    sz = skBitmapGetHighCount(bmap);
    RESULT(sz == 2);

    TEST("skBitmapGetBit");
    rv = skBitmapGetBit(bmap, j);
    RESULT(rv == 1);

    TEST("skBitmapClearBit");
    skBitmapClearBit(bmap, i);
    CHECK_MAP(bmap, i);
    RESULT(1);

    TEST("skBitmapGetHighCount");
    sz = skBitmapGetHighCount(bmap);
    RESULT(sz == 1);

    TEST("skBitmapClearAllBits");
    skBitmapClearAllBits(bmap);
    CHECK_MAP(bmap, (BITMAP_SIZE * 2));
    RESULT(1);

    TEST("skBitmapGetHighCount");
    sz = skBitmapGetHighCount(bmap);
    RESULT(sz == 0);

    TEST("skBitmapGetSize");
    sz = skBitmapGetSize(bmap);
    RESULT(sz == BITMAP_SIZE);

    i = 95;

    TEST("skBitmapSetBit");
    skBitmapSetBit(bmap, i);
    CHECK_MAP(bmap, i);
    RESULT(1);

    TEST("skBitmapGetBit");
    rv = skBitmapGetBit(bmap, i);
    RESULT(rv == 1);

    TEST("skBitmapGetHighCount");
    sz = skBitmapGetHighCount(bmap);
    RESULT(sz == 1);

    TEST("skBitmapGetBit");
    rv = skBitmapGetBit(bmap, j);
    RESULT(rv == 0);

    TEST("skBitmapClearBit");
    skBitmapClearBit(bmap, j);
    CHECK_MAP(bmap, i);
    RESULT(1);

    TEST("skBitmapGetBit");
    rv = skBitmapGetBit(bmap, j);
    RESULT(rv == 0);

    TEST("skBitmapGetBit");
    rv = skBitmapGetBit(bmap, i);
    RESULT(rv == 1);

    TEST("skBitmapGetHighCount");
    sz = skBitmapGetHighCount(bmap);
    RESULT(sz == 1);

    TEST("skBitmapClearBit");
    skBitmapClearBit(bmap, i);
    CHECK_MAP(bmap, (BITMAP_SIZE * 2));
    RESULT(1);

    TEST("skBitmapGetHighCount");
    sz = skBitmapGetHighCount(bmap);
    RESULT(sz == 0);

    i = 0;

    TEST("skBitmapSetBit");
    skBitmapSetBit(bmap, i);
    CHECK_MAP(bmap, i);
    RESULT(1);

    TEST("skBitmapGetBit");
    rv = skBitmapGetBit(bmap, i);
    RESULT(rv == 1);

    TEST("skBitmapGetHighCount");
    sz = skBitmapGetHighCount(bmap);
    RESULT(sz == 1);

    TEST("skBitmapGetBit");
    rv = skBitmapGetBit(bmap, j);
    RESULT(rv == 0);

    TEST("skBitmapClearBit");
    skBitmapClearBit(bmap, j);
    CHECK_MAP(bmap, i);
    RESULT(1);

    TEST("skBitmapGetBit");
    rv = skBitmapGetBit(bmap, j);
    RESULT(rv == 0);

    TEST("skBitmapGetBit");
    rv = skBitmapGetBit(bmap, i);
    RESULT(rv == 1);

    TEST("skBitmapGetHighCount");
    sz = skBitmapGetHighCount(bmap);
    RESULT(sz == 1);

    TEST("skBitmapClearBit");
    skBitmapClearBit(bmap, i);
    CHECK_MAP(bmap, (BITMAP_SIZE * 2));
    RESULT(1);

    TEST("skBitmapGetHighCount");
    sz = skBitmapGetHighCount(bmap);
    RESULT(sz == 0);

    i = BITMAP_SIZE - 1;

    TEST("skBitmapSetBit");
    skBitmapSetBit(bmap, i);
    CHECK_MAP(bmap, i);
    RESULT(1);

    TEST("skBitmapGetBit");
    rv = skBitmapGetBit(bmap, i);
    RESULT(rv == 1);

    TEST("skBitmapGetHighCount");
    sz = skBitmapGetHighCount(bmap);
    RESULT(sz == 1);

    TEST("skBitmapGetBit");
    rv = skBitmapGetBit(bmap, j);
    RESULT(rv == 0);

    TEST("skBitmapClearBit");
    skBitmapClearBit(bmap, j);
    CHECK_MAP(bmap, i);
    RESULT(1);

    TEST("skBitmapGetBit");
    rv = skBitmapGetBit(bmap, j);
    RESULT(rv == 0);

    TEST("skBitmapGetBit");
    rv = skBitmapGetBit(bmap, i);
    RESULT(rv == 1);

    TEST("skBitmapGetHighCount");
    sz = skBitmapGetHighCount(bmap);
    RESULT(sz == 1);

    TEST("skBitmapClearBit");
    skBitmapClearBit(bmap, i);
    CHECK_MAP(bmap, (BITMAP_SIZE * 2));
    RESULT(1);

    TEST("skBitmapGetHighCount");
    sz = skBitmapGetHighCount(bmap);
    RESULT(sz == 0);

    TEST("skBitmapDestroy");
    skBitmapDestroy(&bmap);
    RESULT(bmap == NULL);

    TEST("skBitmapDestroy");
    skBitmapDestroy(&bmap);
    RESULT(bmap == NULL);

    TEST("skBitmapCreate");
    rv = skBitmapCreate(&bmap, BITMAP_SIZE);
    CHECK_MAP(bmap, (BITMAP_SIZE * 2));
    RESULT(rv == 0 && bmap != NULL);

    TEST("skBitmapGetSize");
    sz = skBitmapGetSize(bmap);
    RESULT(sz == BITMAP_SIZE);

    TEST("skBitmapGetHighCount");
    sz = skBitmapGetHighCount(bmap);
    RESULT(sz == 0);

    TEST("skBitmapDestroy");
    skBitmapDestroy(&bmap);
    RESULT(bmap == NULL);

    return 0;
}
