/*
** Copyright (C) 2006 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
** 
** Use of the SILK system and related source code is subject to the terms 
** of the following licenses:
** 
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
** 
** NO WARRANTY
** 
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
** DELIVERABLES UNDER THIS LICENSE.
** 
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
** Mellon University, its trustees, officers, employees, and agents from 
** all claims or demands made against them (and any related losses, 
** expenses, or attorney's fees) arising out of, or relating to Licensee's 
** and/or its sub licensees' negligent use or willful misuse of or 
** negligent conduct or willful misconduct regarding the Software, 
** facilities, or other rights or assistance granted by Carnegie Mellon 
** University under this License, including, but not limited to, any 
** claims of product liability, personal injury, death, damage to 
** property, or violation of any laws or regulations.
** 
** Carnegie Mellon University Software Engineering Institute authored 
** documents are sponsored by the U.S. Department of Defense under 
** Contract F19628-00-C-0003. Carnegie Mellon University retains 
** copyrights in all material produced under this contract. The U.S. 
** Government retains a non-exclusive, royalty-free license to publish or 
** reproduce these documents, or allow others to do so, for U.S. 
** Government purposes only pursuant to the copyright license under the 
** contract clause at 252.227.7013.
** 
** @OPENSOURCE_HEADER_END@
*/

/*
**  sku-compat.c
**
**    Function 
**
*/


#include "silk.h"

RCSIDENT("$SiLK: sku-compat.c 5718 2006-11-16 17:06:17Z mthomas $");

#include "utils.h"


#if !HAVE_IMAXDIV_T
/* Compute quotient and remainder in one structure like div(), but
 * with intmax_t's instead of int's. */
imaxdiv_t imaxdiv(intmax_t numer, intmax_t denom)
{
    imaxdiv_t res;
    res.quot = numer / denom;
    res.rem = numer % denom;
    return res;
}
#endif /* !HAVE_IMAXDIV_T */


#if !HAVE_TIMEGM
/* Inverse of gmtime(); convert a struct tm to time_t. */
time_t timegm(struct tm *tm)
{
    /* Some OSes will put this string directly into the environment--as
     * opposed to making a copy of it--so we make it static. */
    static char envbuf[1024];
    time_t ret;
    char *tz;

    /* get old timezone */
    tz = getenv("TZ");

    /* make TZ empty (UTC) and set timezone there */
    if (putenv("TZ=")) {
        skAppPrintErr("timegm(): Out of memory!");
        exit(EXIT_FAILURE);
    }
    tzset();

    /* do the work */
    ret = mktime(tm);

    /* restore TZ and timezone. Magic 6 is space we allow for "TZ=" */
    if (tz && (strlen(tz) < (sizeof(envbuf) - 6))) {
        sprintf(envbuf, "TZ=%s", tz);
        putenv(envbuf);
    } else {
        /* unset */
        putenv("TZ");
    }
    tzset();

    return ret;
}
#endif /* !HAVE_TIMEGM */
