/*
** Copyright (C) 2006-2007 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
** 
** Use of the SILK system and related source code is subject to the terms 
** of the following licenses:
** 
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
** 
** NO WARRANTY
** 
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
** DELIVERABLES UNDER THIS LICENSE.
** 
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
** Mellon University, its trustees, officers, employees, and agents from 
** all claims or demands made against them (and any related losses, 
** expenses, or attorney's fees) arising out of, or relating to Licensee's 
** and/or its sub licensees' negligent use or willful misuse of or 
** negligent conduct or willful misconduct regarding the Software, 
** facilities, or other rights or assistance granted by Carnegie Mellon 
** University under this License, including, but not limited to, any 
** claims of product liability, personal injury, death, damage to 
** property, or violation of any laws or regulations.
** 
** Carnegie Mellon University Software Engineering Institute authored 
** documents are sponsored by the U.S. Department of Defense under 
** Contract F19628-00-C-0003. Carnegie Mellon University retains 
** copyrights in all material produced under this contract. The U.S. 
** Government retains a non-exclusive, royalty-free license to publish or 
** reproduce these documents, or allow others to do so, for U.S. 
** Government purposes only pursuant to the copyright license under the 
** contract clause at 252.227.7013.
** 
** @OPENSOURCE_HEADER_END@
*/

/*
**  Simple tester for the skpolldir library
**
*/

#include "silk.h"

RCSIDENT("$SiLK: polldir-test.c 6272 2007-02-13 20:04:22Z mwd $");

#include "skpolldir.h"
#include "utils.h"

int main (int argc, char **argv)
{
    skPollDirQueue_t queue;
    skPollDir_t pd;
    const char *dirname;
    uint32_t interval = 10;
    char path[PATH_MAX];
    char *file;
    
    /* register the application */
    skAppRegister(argv[0]);

    /* make certain there are enough args.  If first arg begins with
     * hyphen, print usage. */
    if (argc < 2 || argc > 3 || argv[1][0] == '-') {
        fprintf(stderr, "Usage: %s <dirname> [<poll-interval>]\n", 
                skAppName());
        return EXIT_FAILURE;
    }

    /* get directory to poll */
    dirname = argv[1];
    if (!dirExists(dirname)) {
        skAppPrintErr("polling dir '%s' does not exist", dirname);
        return EXIT_FAILURE;
    }

    /* get interval if given */
    if (argc == 3) {
        int rv = skStringParseUint32(&interval, argv[2], 1, 0);
        if (rv != 0) {
            skAppPrintErr("failed to parse '%s' as a positive integer", 
                          argv[2]);
            return EXIT_FAILURE;
        }
    }

    queue = skPollDirQueueCreate();
    pd = skPollDirCreate(dirname, interval, queue);
    if (queue == NULL || pd == NULL) {
        skAppPrintErr("failed to set up polling for directory %s", dirname);
        return EXIT_FAILURE;
    }

    while (PDERR_NONE == skPollDirGetNextFile(queue, path, &file)) {
        printf("%s\n", file);
    }

    skAppUnregister();
    return EXIT_SUCCESS;
}


    


/*
** Local variables:
** mode:c
** indent-tabs-mode:nil
** c-basic-offset:4
** End:
*/
