/*
** Copyright (C) 2004-2007 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
**
** Use of the SILK system and related source code is subject to the terms
** of the following licenses:
**
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
**
** NO WARRANTY
**
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE,
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT,
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES.
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE
** DELIVERABLES UNDER THIS LICENSE.
**
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie
** Mellon University, its trustees, officers, employees, and agents from
** all claims or demands made against them (and any related losses,
** expenses, or attorney's fees) arising out of, or relating to Licensee's
** and/or its sub licensees' negligent use or willful misuse of or
** negligent conduct or willful misconduct regarding the Software,
** facilities, or other rights or assistance granted by Carnegie Mellon
** University under this License, including, but not limited to, any
** claims of product liability, personal injury, death, damage to
** property, or violation of any laws or regulations.
**
** Carnegie Mellon University Software Engineering Institute authored
** documents are sponsored by the U.S. Department of Defense under
** Contract F19628-00-C-0003. Carnegie Mellon University retains
** copyrights in all material produced under this contract. The U.S.
** Government retains a non-exclusive, royalty-free license to publish or
** reproduce these documents, or allow others to do so, for U.S.
** Government purposes only pursuant to the copyright license under the
** contract clause at 252.227.7013.
**
** @OPENSOURCE_HEADER_END@
*/

/*
**  Configuration file parser
**
**  Sep 2004  Dave Svoboda
**
*/


#include "silk.h"

RCSIDENT("$SiLK: cfg_config.c 7345 2007-05-30 20:39:09Z mthomas $");

#include "cfg_config.h"


cfg_err_t cfg_map_free_destroy(void *x)
{
    return cfg_map_destroy(CFG_CAST(cfg_map_t, x), cfg_array_path_destroy);
}

cfg_err_t cfg_array_daterange_destroy(void *x)
{
    return cfg_array_destroy( x, cfg_daterange_destroy);
}

cfg_err_t cfg_array_path_print(
    FILE       *f,
    const void *x,
    int         indent)
{
    return cfg_array_print( f, x, indent, cfg_path_print);
}

cfg_err_t cfg_map_path_print(
    FILE       *f,
    const void *x,
    int         indent)
{
    return cfg_map_print( f, x, indent, cfg_array_path_print);
}

cfg_err_t cfg_array_path_destroy(void *x)
{
    return cfg_array_destroy(x, cfg_path_destroy);
}

cfg_err_t cfg_array_daterange_print(
    FILE       *f,
    const void *x,
    int         indent)
{
    return cfg_array_print( f, x, indent, cfg_daterange_print);
}


/* Returns true if date is within any of the given ranges, or if the
   ranges are NULL */
int cfg_date_in_ranges(
    const cfg_date_t   *date,
    const cfg_array_t  *ranges)
{
    int i;

    if (ranges == NULL) {
        return 1;
    }
    for (i = 0; i < cfg_get_array_length( ranges); i++) {
        if (cfg_date_in_range( CFG_CAST( cfg_daterange_t,
                                         cfg_get_array_element( ranges, i)),
                               date))
        {
            return 1;
        }
    }
    return 0;
}


/** Now for the config functions **/


cfg_err_t cfg_config_create(cfg_config_t **x)
{
    cfg_config_t *result;

    CFG_ALLOCATE( cfg_config_t, result, 1);
    CFG_CHECK( cfg_map_create( &(cfg_get_config_properties(result))));
    CFG_CHECK( cfg_map_create( &(cfg_get_config_sensors(result))));
    CFG_CHECK( cfg_map_create( &(cfg_get_config_flowlabels(result))));
    CFG_CHECK( cfg_map_create( &(cfg_get_config_dates(result))));
    CFG_CHECK( cfg_map_create( &(cfg_get_config_domains(result))));
    *x = result;
    return CFG_OK;
}


cfg_err_t cfg_config_destroy(cfg_config_t *x)
{
    CFG_CHECK( cfg_map_destroy( cfg_get_config_properties(x),
                                cfg_free));
    CFG_CHECK( cfg_map_destroy( cfg_get_config_sensors(x),
                                cfg_array_free_destroy));
    CFG_CHECK( cfg_map_destroy( cfg_get_config_flowlabels(x),
                                cfg_array_free_destroy));
    CFG_CHECK( cfg_map_destroy( cfg_get_config_dates(x),
                                cfg_array_daterange_destroy));
    CFG_CHECK( cfg_map_destroy( cfg_get_config_domains(x),
                                cfg_map_free_destroy));
    FREE( x);
    return CFG_OK;
}


cfg_err_t cfg_config_print(
    FILE               *out,
    const cfg_config_t *x,
    int                 indent)
{
    /* Print format is: (Configuration: Flowlabels: ... Sensors: ...
       Properties: ... Dates: ... Domains: ...) */
    CFG_FPRINTF((out, "(Configuration:\n Flowlabels: "));
    CFG_CHECK( cfg_map_print( out, cfg_get_config_flowlabels(x), indent,
                              cfg_array_string_print));
    CFG_FPRINTF((out, "\n Sensors: "));
    CFG_CHECK( cfg_map_print( out, cfg_get_config_sensors(x), indent,
                              cfg_array_string_print));
    CFG_FPRINTF((out, "\n Properties: "));
    CFG_CHECK( cfg_map_print( out, cfg_get_config_properties(x), indent,
                              cfg_string_print));
    CFG_FPRINTF((out, "\n Dates: "));
    CFG_CHECK( cfg_map_print( out, cfg_get_config_dates(x), indent,
                              cfg_array_daterange_print));
    CFG_FPRINTF((out, "\n Domains: "));
    CFG_CHECK( cfg_map_print( out, cfg_get_config_domains(x), indent,
                              cfg_map_path_print));
    CFG_FPRINTF(( out, ")\n"));
    return CFG_OK;
}


const cfg_array_t *cfg_config_lookup(
    const cfg_config_t *x,
    const char         *domain,
    const char         *path)
{
    cfg_map_t *domain_map = cfg_lookup_config_domain( x, domain);

    return (domain_map == NULL) ? NULL :
        cfg_lookup_domain_paths( domain_map, path);
}


cfg_array_t *cfg_config_lookup_date(
    const cfg_config_t *x,
    const char         *domain,
    const char         *path,
    time_t              timestamp)
{
    int i;
    const cfg_array_t *paths = cfg_config_lookup( x, domain, path);
    cfg_array_t *result = NULL;
    cfg_date_t *date;

    cfg_date_create(&date, timestamp);
    if (paths != NULL) {
        cfg_array_create( &result);
        for (i = 0; i < cfg_get_array_length( paths); i++) {
            if (cfg_date_in_ranges( date, cfg_get_path_dateranges(
                  CFG_CAST( cfg_path_t, cfg_get_array_element( paths, i)))))
            {
                cfg_path_t *new_path;
                cfg_path_clone(CFG_CAST(void*, &new_path),
                               cfg_get_array_element( paths, i));
                /* cfg_path_print( stdout, new_path); */
                cfg_array_add( result, new_path);
            }
        }
    }
    return result;
}


const char *cfg_config_get_property(
    const cfg_config_t *x,
    const char         *property_name)
{
    return cfg_map_assoc( cfg_get_config_properties(x), property_name);
}


/*
** Local Variables:
** mode:c
** indent-tabs-mode:nil
** c-basic-offset:4
** End:
*/
