/*
** Copyright (C) 2004-2007 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
**
** Use of the SILK system and related source code is subject to the terms
** of the following licenses:
**
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
**
** NO WARRANTY
**
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE,
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT,
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES.
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE
** DELIVERABLES UNDER THIS LICENSE.
**
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie
** Mellon University, its trustees, officers, employees, and agents from
** all claims or demands made against them (and any related losses,
** expenses, or attorney's fees) arising out of, or relating to Licensee's
** and/or its sub licensees' negligent use or willful misuse of or
** negligent conduct or willful misconduct regarding the Software,
** facilities, or other rights or assistance granted by Carnegie Mellon
** University under this License, including, but not limited to, any
** claims of product liability, personal injury, death, damage to
** property, or violation of any laws or regulations.
**
** Carnegie Mellon University Software Engineering Institute authored
** documents are sponsored by the U.S. Department of Defense under
** Contract F19628-00-C-0003. Carnegie Mellon University retains
** copyrights in all material produced under this contract. The U.S.
** Government retains a non-exclusive, royalty-free license to publish or
** reproduce these documents, or allow others to do so, for U.S.
** Government purposes only pursuant to the copyright license under the
** contract clause at 252.227.7013.
**
** @OPENSOURCE_HEADER_END@
*/

/*
**  A simple set data type
**
**  Sep 2004  Dave Svoboda
**
*/

#include "silk.h"

RCSIDENT("$SiLK: cfg_map.c 7345 2007-05-30 20:39:09Z mthomas $");

#include "cfg_config.h"


/** Array of string functions **/


cfg_err_t cfg_array_free_destroy(void *x)
{
    return cfg_array_destroy( x, cfg_free);
}


cfg_err_t cfg_array_string_print(
    FILE       *f,
    const void *x,
    int         indent)
{
    return cfg_array_print( f, x, indent, cfg_string_print);
}


/** Map functions **/


cfg_err_t cfg_map_create(cfg_map_t **x)
{
    cfg_map_t *result;

    CFG_ALLOCATE( cfg_map_t, result, 1);
    CFG_CHECK( cfg_array_create( &(result->keys)));
    CFG_CHECK( cfg_array_create( &(result->values)));
    *x = result;
    return CFG_OK;
}


cfg_err_t cfg_map_destroy(
    cfg_map_t              *x,
    cfg_destroy_function_t  destroy_function)
{
    CFG_CHECK( cfg_array_destroy( x->keys, cfg_free));
    CFG_CHECK( cfg_array_destroy( x->values, destroy_function));
    FREE( x);
    return CFG_OK;
}


cfg_err_t cfg_map_add(
    cfg_map_t  *x,
    char       *key,
    void       *value)
{
    int i = cfg_array_find( x->keys, key, cfg_string_equality);

    if (i == -1) {
        /* FIXME: A good map would have keys in sorted order. We are just
           being lazy and inserting them as we get 'em */
        CFG_CHECK( cfg_array_add( x->keys, key));
        CFG_CHECK( cfg_array_add( x->values, value));
    } else {
        /* FIXME: We should free the old value, but don't know its type */
        x->values->elements[i] = value;
        return CFG_DATA_OVERWRITTEN;
    }
    return CFG_OK;
}


void *cfg_map_assoc(
    const cfg_map_t    *x,
    const char         *key)
{
    int i = cfg_array_find( x->keys, key, cfg_string_equality);

    return ((i == -1) ? NULL : cfg_get_map_value( x, i));
}


/*
** Local Variables:
** mode:c
** indent-tabs-mode:nil
** c-basic-offset:4
** End:
*/
