/*
** Copyright (C) 2004-2007 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
**
** Use of the SILK system and related source code is subject to the terms
** of the following licenses:
**
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
**
** NO WARRANTY
**
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE,
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT,
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES.
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE
** DELIVERABLES UNDER THIS LICENSE.
**
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie
** Mellon University, its trustees, officers, employees, and agents from
** all claims or demands made against them (and any related losses,
** expenses, or attorney's fees) arising out of, or relating to Licensee's
** and/or its sub licensees' negligent use or willful misuse of or
** negligent conduct or willful misconduct regarding the Software,
** facilities, or other rights or assistance granted by Carnegie Mellon
** University under this License, including, but not limited to, any
** claims of product liability, personal injury, death, damage to
** property, or violation of any laws or regulations.
**
** Carnegie Mellon University Software Engineering Institute authored
** documents are sponsored by the U.S. Department of Defense under
** Contract F19628-00-C-0003. Carnegie Mellon University retains
** copyrights in all material produced under this contract. The U.S.
** Government retains a non-exclusive, royalty-free license to publish or
** reproduce these documents, or allow others to do so, for U.S.
** Government purposes only pursuant to the copyright license under the
** contract clause at 252.227.7013.
**
** @OPENSOURCE_HEADER_END@
*/

/*
**  A simple path data type
**
**  Sep 2004  Dave Svoboda
**
*/

#include "silk.h"

RCSIDENT("$SiLK: cfg_path.c 7345 2007-05-30 20:39:09Z mthomas $");

#include "cfg_config.h"


/** Path functions **/


cfg_err_t cfg_path_create(
    cfg_path_t    **x,
    char           *sensor,
    char           *flowlabel,
    cfg_array_t    *dateranges)
{
    cfg_path_t *result;
    char *new_sensor = NULL;
    char *new_flowlabel = NULL;
    cfg_array_t *new_dateranges = NULL;

    CFG_ALLOCATE( cfg_path_t, result, 1);
    CFG_CHECK( cfg_string_clone( CFG_CAST_PTR( void, new_sensor),
                                 sensor));
    CFG_CHECK( cfg_string_clone( CFG_CAST_PTR( void, new_flowlabel),
                                 flowlabel));
    if (dateranges != NULL) {
        CFG_CHECK( cfg_array_clone( &new_dateranges,
                                    dateranges,
                                    cfg_daterange_clone));
    }
    cfg_get_path_sensor( result) = new_sensor;
    cfg_get_path_flowlabel( result) = new_flowlabel;
    cfg_get_path_dateranges( result) = new_dateranges;
    *x = result;
    return CFG_OK;
}


cfg_err_t cfg_path_clone(
    void      **result,
    const void *x)
{
    cfg_path_t *p = CFG_CAST( cfg_path_t, x);

    return cfg_path_create( CFG_CAST_PTR( cfg_path_t, *result),
                            cfg_get_path_sensor(p),
                            cfg_get_path_flowlabel(p),
                            cfg_get_path_dateranges(p));
}


cfg_err_t cfg_path_destroy(void *x)
{
    cfg_path_t *p = CFG_CAST( cfg_path_t, x);

    FREE( cfg_get_path_sensor(p));
    FREE( cfg_get_path_flowlabel(p));
    if (cfg_get_path_dateranges(p) != NULL) {
        cfg_array_destroy( cfg_get_path_dateranges(p),
                           cfg_daterange_destroy);
    }
    FREE(p);
    return CFG_OK;
}


cfg_err_t cfg_path_print(
    FILE       *out,
    const void *x,
    int         indent)
{
    /* Print format is: <sensor>.<flowlabel> [<daterange>]*/
    cfg_path_t *path = CFG_CAST( cfg_path_t, x);

    CFG_CHECK( cfg_string_print( out, cfg_get_path_sensor(path), indent));
    CFG_FPRINTF(( out, "."));
    CFG_CHECK( cfg_string_print( out, cfg_get_path_flowlabel(path), indent));
    if (cfg_get_path_dateranges(path) != NULL) {
        CFG_FPRINTF(( out, " "));
        CFG_CHECK( cfg_array_print( out, cfg_get_path_dateranges(path), indent,
                                    cfg_daterange_print));
    }
    return CFG_OK;
}


/*
** Local Variables:
** mode:c
** indent-tabs-mode:nil
** c-basic-offset:4
** End:
*/
