/*
** Copyright (C) 2004-2007 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
**
** Use of the SILK system and related source code is subject to the terms
** of the following licenses:
**
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
**
** NO WARRANTY
**
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE,
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT,
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES.
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE
** DELIVERABLES UNDER THIS LICENSE.
**
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie
** Mellon University, its trustees, officers, employees, and agents from
** all claims or demands made against them (and any related losses,
** expenses, or attorney's fees) arising out of, or relating to Licensee's
** and/or its sub licensees' negligent use or willful misuse of or
** negligent conduct or willful misconduct regarding the Software,
** facilities, or other rights or assistance granted by Carnegie Mellon
** University under this License, including, but not limited to, any
** claims of product liability, personal injury, death, damage to
** property, or violation of any laws or regulations.
**
** Carnegie Mellon University Software Engineering Institute authored
** documents are sponsored by the U.S. Department of Defense under
** Contract F19628-00-C-0003. Carnegie Mellon University retains
** copyrights in all material produced under this contract. The U.S.
** Government retains a non-exclusive, royalty-free license to publish or
** reproduce these documents, or allow others to do so, for U.S.
** Government purposes only pursuant to the copyright license under the
** contract clause at 252.227.7013.
**
** @OPENSOURCE_HEADER_END@
*/
#ifndef _RWFILTERSTATS_H
#define _RWFILTERSTATS_H

#include "silk.h"

RCSIDENTVAR(rcsID_RWFILTERSTATS_H, "$SiLK: rwfilterstats.h 7311 2007-05-25 19:07:53Z mthomas $");

/*
**  rwfilterstats.h
**
**  Header for the rwfilterstats library.
**
*/



/* for counting the flows, packets, and bytes */
typedef struct {
    uint64_t  flows;
    uint64_t  pkts;
    uint64_t  bytes;
} rec_count_t;

/* holds filter-statistics data */
typedef struct {
    genericHeader   gHdr;           /* generic header */
    rec_count_t     rCount;         /* recs */
    rec_count_t     pCount;         /* passed */
    uint32_t        fileCount;      /* files */
    uint8_t         volumeStats;    /* whether to print detailed stats */
} filterStatisticsV2_t;



extern void filterStatisticsDump(FILE *fh, const filterStatisticsV2_t *hdr);
/*
 *    Print the statistics in the structure 'hdr' to file handle 'fh'.
 *    Write the statistics in the correct form depending on whether we
 *    are running in a cluster environment.
 */


extern int  filterStatisticsInit(filterStatisticsV2_t *hdr);
/*
 *    Initialize the filter data structure 'hdr'.  The caller is
 *    responsible for the allocatation of 'hdr'.  Returns 0 unless
 *    'hdr' is NULL, in which case -1 is returned.
 */

extern int  filterStatisticsAddFromFile(
    filterStatisticsV2_t   *sum,
    const genericHeader    *gHdr,
    int                     fd);
/*
 *    Add to the filter statistics in 'sum' the filter statistics that
 *    are read from the file descriptor 'fd'.  The routine assumes the
 *    file is open and its generic header has already been read into
 *    the variable 'gHdr'.  Returns 0 on success; -1 if the data could
 *    not be read from the file.
 */

int filterStatisticsAsString(
    char                       *buf,
    size_t                      buf_len,
    const filterStatisticsV2_t *hdr);
/*
 *    Produces a textual (human readable) version of the filter
 *    statistics contained in 'hdr'; the text is stored in 'buf' whose
 *    size is given by 'buf_len'.  The routine returns the number of
 *    bytes of 'buf' that were used---i.e., the strlen() of 'buf'.
 *    Returns -1 if the input was incorrect or if the 'buf' is too
 *    small to hold the statistics.
 */




#endif /* _RWFILTERSTATS_H */


/*
** Local Variables:
** mode:c
** indent-tabs-mode:nil
** c-basic-offset:4
** End:
*/
