/*
** Copyright (C) 2004-2006 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
** 
** Use of the SILK system and related source code is subject to the terms 
** of the following licenses:
** 
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
** 
** NO WARRANTY
** 
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
** DELIVERABLES UNDER THIS LICENSE.
** 
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
** Mellon University, its trustees, officers, employees, and agents from 
** all claims or demands made against them (and any related losses, 
** expenses, or attorney's fees) arising out of, or relating to Licensee's 
** and/or its sub licensees' negligent use or willful misuse of or 
** negligent conduct or willful misconduct regarding the Software, 
** facilities, or other rights or assistance granted by Carnegie Mellon 
** University under this License, including, but not limited to, any 
** claims of product liability, personal injury, death, damage to 
** property, or violation of any laws or regulations.
** 
** Carnegie Mellon University Software Engineering Institute authored 
** documents are sponsored by the U.S. Department of Defense under 
** Contract F19628-00-C-0003. Carnegie Mellon University retains 
** copyrights in all material produced under this contract. The U.S. 
** Government retains a non-exclusive, royalty-free license to publish or 
** reproduce these documents, or allow others to do so, for U.S. 
** Government purposes only pursuant to the copyright license under the 
** contract clause at 252.227.7013.
** 
** @OPENSOURCE_HEADER_END@
*/
#ifndef _STREAM_CACHE_H
#define _STREAM_CACHE_H

#include "silk.h"

/* RCSIDENTVAR(rcsID_STREAM_CACHE_H, "$SiLK: stream-cache.h 2920 2006-02-06 18:02:36Z mthomas $"); */

/*
**  stream-cache.h
**
**  A simple interface for maintaining a list of open file handles so
**  we can avoid a lot of open/close cycles.  File handles are indexed
**  by the timestamp of the file and the type and version of data they
**  contain.
*/

#include "rwpack.h"


struct _streamCache;
typedef struct _streamCache streamCache_t;


/*
 * cache = skStreamCacheCreate(type_count, size);
 *
 * Creates a streamcache.  A stream cache is actually an array of
 * caches, indexed by the stream type.  Returns NULL on error.
 */
extern streamCache_t *skStreamCacheCreate(int type_count, int size);

/*
 * skStreamCacheDestroy(cache);
 *
 * Close all streams and free all memory associated with the streams.
 * Free the memory associated with the cache.  The cache pointer is
 * invalid after a call to these function.
 */
extern void skStreamCacheDestroy(streamCache_t *c);

/*
 * skStreamCacheFlush(cache);
 *
 * Call fflush() on the file handles associated with all the streams
 * in the cache.
 */
extern void skStreamCacheFlush(streamCache_t *c);


/*
 * skStreamCacheCloseAll(cache);
 *
 *   Close all handles associated with all the streams in the cache.
 */
void skStreamCacheCloseAll(streamCache_t *cache);


/*
 * skStreamCacheAdd(cache, type, stream);
 *
 * Note the cache will own stream memory and will free it when the
 * stream falls off the end of the cache, or when the cache is
 * destroyed.
 */
extern void skStreamCacheAdd(
    streamCache_t   *c,
    int              streamType,
    rwIOStruct      *s);

/*
 * stream = skStreamCacheLookup(cache, type, timestamp, sensor_id);
 *
 *   Search the cache for the stream that has the specified type,
 *   time, and sensor_id.  If found, return the stream.  If not found,
 *   return NULL.
 */
extern rwIOStruct *skStreamCacheLookup(
    streamCache_t  *c,
    int             stream_type,
    uint32_t        time_stamp,
    sensorID_t      sensor_id);


#endif /* _STREAM_CACHE_H */
