/*
** Copyright (C) 2004-2007 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
**
** Use of the SILK system and related source code is subject to the terms
** of the following licenses:
**
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
**
** NO WARRANTY
**
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE,
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT,
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES.
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE
** DELIVERABLES UNDER THIS LICENSE.
**
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie
** Mellon University, its trustees, officers, employees, and agents from
** all claims or demands made against them (and any related losses,
** expenses, or attorney's fees) arising out of, or relating to Licensee's
** and/or its sub licensees' negligent use or willful misuse of or
** negligent conduct or willful misconduct regarding the Software,
** facilities, or other rights or assistance granted by Carnegie Mellon
** University under this License, including, but not limited to, any
** claims of product liability, personal injury, death, damage to
** property, or violation of any laws or regulations.
**
** Carnegie Mellon University Software Engineering Institute authored
** documents are sponsored by the U.S. Department of Defense under
** Contract F19628-00-C-0003. Carnegie Mellon University retains
** copyrights in all material produced under this contract. The U.S.
** Government retains a non-exclusive, royalty-free license to publish or
** reproduce these documents, or allow others to do so, for U.S.
** Government purposes only pursuant to the copyright license under the
** contract clause at 252.227.7013.
**
** @OPENSOURCE_HEADER_END@
*/

/*
**  Test the parsePath() function in naming.c.  For each argv[i]
**  on command line, passes it to parsePath() and prints result.
**
*/


#include "silk.h"

RCSIDENT("$SiLK: test-naming.c 6081 2007-01-22 19:25:22Z mthomas $");

#include "sendrcv.h"


int main(int argc, char **argv)
{
    int i;
    int no_slash_warning = 0;
    file_names_t fns;

    /* register the application */
    skAppRegister(argv[0]);

    namingSetDataRootdir("/data");

    for (i = 1; i < argc; ++i) {
        /* only allow names with a slash */
        if (NULL == strchr(argv[i], '/')) {
            /* only print warning one time */
            if (0 == no_slash_warning) {
                no_slash_warning = 1;
                fprintf(stderr, "input names must contain a '/'\n");
            }
            continue;
        }

        memset(&fns, sizeof(file_names_t), 0);
        fns.input_path = argv[i];
        fprintf(stderr, "'%s' => ", fns.input_path);
        if (parsePath(&fns)) {
            fprintf(stderr, "PARSING ERROR!!\n");
        } else {
            fprintf(stderr, ("\n"
                             "\tINPUT PARTS:   '%s'\n"
                             "\t               '%s'\n"
                             "\tOUTPUT ALL:    '%s'\n"
                             "\tOUTPUT PARTS:  '%s'\n"
                             "\t               '%s'\n"
                             "\t               '%s'\n"),
                    fns.input_dirname, fns.input_basename,
                    fns.output_path,
                    fns.data_rootdir, fns.relative_dir, fns.output_file);
        }
    }

    skAppUnregister();
    return 0;
}


/*
** Local variables:
** mode:c
** indent-tabs-mode:nil
** c-basic-offset:4
** End:
*/
